package cn.com.qmth.examcloud.commons.util;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.commons.io.IOUtils;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;

/**
 * 类注释
 *
 * @author WANGWEI
 */
public class HttpClientUtil {
	private static final ExamCloudLog LOGGER = ExamCloudLogFactory.getLog(HttpClientUtil.class);

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param url
	 * @param data
	 * @param timeout
	 * @return
	 */
	public static String post(String url, String data, int timeout) {
		StringBuilder responseBuilder = null;
		BufferedReader reader = null;
		OutputStreamWriter wr = null;

		URL u = null;
		try {
			u = new URL(url);
			HttpURLConnection conn = (HttpURLConnection) u.openConnection();

			conn.setDoOutput(true);
			conn.setDoInput(true);

			conn.setRequestMethod("POST");

			if (null != data) {
				conn.setConnectTimeout(timeout);
				wr = new OutputStreamWriter(conn.getOutputStream());

				wr.write(data);
				wr.flush();
			}

			reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
			responseBuilder = new StringBuilder();
			String line = null;
			while ((line = reader.readLine()) != null) {
				responseBuilder.append(line).append("\n");
			}
			return responseBuilder.toString();
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(wr);
			IOUtils.closeQuietly(reader);
		}
	}

	/**
	 * 关闭流
	 *
	 * @author WANGWEI
	 * @param resp
	 */
	public static void close(CloseableHttpResponse resp) {
		if (null != resp) {
			try {
				EntityUtils.consumeQuietly(resp.getEntity());
				resp.close();
			} catch (Exception e) {
				LOGGER.error("fail to close http response stream.", e);
			}
		}
	}

	public static byte[] get(String url) {
		CloseableHttpClient httpclient = HttpClientPool.getHttpClient();
		HttpGet get = new HttpGet(url);
		get.setConfig(RequestConfig.custom().setConnectTimeout(10000).build());
		CloseableHttpResponse response = null;
		try {
			response = httpclient.execute(get);
			InputStream in = response.getEntity().getContent();

			byte[] byteArray = IOUtil.toLimitedByteArray(in, 1048576);
			return byteArray;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}